package ru.cft.task.restClient;

import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.stage.Stage;

import java.io.*;
import java.util.Properties;

public class ConfigController {
    private Properties props;
    private InputStream conf_file = null;
    private OutputStream save_file = null;
    private static final String CONF_FILE = "config.properties";

    @FXML
    private TextField url;

    @FXML
    private TextField port;

    @FXML
    private Button cancel;

    @FXML
    private Button save;

    @FXML
    public void initialize() {
        props = new Properties();
        try {
            File cf = new File(CONF_FILE);
            if (!cf.exists()) {
                cf.createNewFile();
                Utils.showAlert("warn", "Нет сохраненного файла настроек.\n" +
                        "Будут установлены умолчательные параметры.\n" +
                        "Необходимо изменить и сохранить.");
            }
            conf_file = new FileInputStream(CONF_FILE);
            props.load(conf_file);
            url.setText(props.getProperty("url", "http://localhost"));
            port.setText(props.getProperty("port", "8080"));
        } catch (IOException ex) {
            Utils.showAlert("error", ex.getMessage());
        } finally {
            if (conf_file != null) {
                try {
                    conf_file.close();
                } catch (IOException ex) {
                    Utils.showAlert("error", ex.getMessage());
                }
            }
        }
    }

    public void saveAction() {
        try {
            save_file = new FileOutputStream(CONF_FILE);
            props.setProperty("url", url.getText());
            props.setProperty("port", port.getText());
            props.store(save_file, "Rest server config");
            Stage stage = (Stage) save.getScene().getWindow();
            stage.close();
        } catch (IOException ex) {
            Utils.showAlert("error", ex.getMessage());
        } finally {
            if (save_file != null) {
                try {
                    save_file.close();
                } catch (IOException ex) {
                    System.err.println(ex.getMessage());
                }
            }
        }
    }

    public void cancelAction() {
        Stage stage = (Stage) cancel.getScene().getWindow();
        stage.close();
    }

    public static String getConfigFile() {
        return CONF_FILE;
    }
}
