package ru.cft.task.restClient;

import com.fasterxml.jackson.databind.ObjectMapper;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.stage.Stage;
import org.springframework.boot.autoconfigure.security.SecurityProperties;
import org.springframework.http.*;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class RestActionsController {
    @FXML
    private TextField id;

    @FXML
    private TextField name;

    @FXML
    private TextField email;

    @FXML
    private Button cancel;

    private String getRestServerUrl() {
        Properties props = new Properties();
        InputStream conf_file = null;
        String url = "";
        try {
            conf_file = new FileInputStream(ConfigController.getConfigFile());
            props.load(conf_file);
            url = props.getProperty("url", "http://localhost");
            url += ":" + props.getProperty("port", "8080");
        } catch (IOException ex) {
            Utils.showAlert("error", ex.getMessage());
        } finally {
            if (conf_file != null) {
                try {
                    conf_file.close();
                } catch (IOException ex) {
                    Utils.showAlert("error", ex.getMessage());
                }
            }
            return url;
        }
    }

    public void cancelAction() {
        Stage stage = (Stage) cancel.getScene().getWindow();
        stage.close();
    }

    public void createAction() {
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> map = new LinkedMultiValueMap<String, String>();
        map.add("name", name.getText());
        map.add("email", email.getText());

        HttpEntity<MultiValueMap<String, String>> entity = new HttpEntity<MultiValueMap<String, String>>(map, headers);
        try {
            ResponseEntity<EmailRecord> response = restTemplate.exchange(getRestServerUrl(), HttpMethod.POST, entity, EmailRecord.class);
            if (response.getStatusCode() == HttpStatus.OK) {
                EmailRecord emailRecord = response.getBody();
                id.setText(String.valueOf(emailRecord.getId()));
            }
        } catch (HttpStatusCodeException ex) {
            try {
                ObjectMapper objectMapper = new ObjectMapper();
                ErrorResponse err = objectMapper.readValue(ex.getResponseBodyAsString(), ErrorResponse.class);
                Utils.showAlert("error", err.getMessage());
            } catch (IOException e) {
                Utils.showAlert("error", e.getMessage());
            }
        }
    }

    public void readAction() {
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        headers.set("Accept", MediaType.APPLICATION_JSON_VALUE);
        UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(getRestServerUrl());
        if (!id.getText().isEmpty()) {
            builder.queryParam("id", id.getText());
        } else if (!email.getText().isEmpty()) {
            builder.queryParam("email", email.getText());
        } else {
            builder.queryParam("name", name.getText());
        }

        HttpEntity<?> entity = new HttpEntity<>(headers);
        try {
            ResponseEntity<EmailRecord> response = restTemplate.exchange(builder.build().encode().toUri(), HttpMethod.GET, entity, EmailRecord.class);
            if (response.getStatusCode() == HttpStatus.OK) {
                EmailRecord emailRecord = response.getBody();
                id.setText(String.valueOf(emailRecord.getId()));
                name.setText(emailRecord.getName());
                email.setText(emailRecord.getEmail());
            }
        } catch (HttpStatusCodeException ex) {
            try {
                ObjectMapper objectMapper = new ObjectMapper();
                ErrorResponse err = objectMapper.readValue(ex.getResponseBodyAsString(), ErrorResponse.class);
                Utils.showAlert("error", err.getMessage());
            } catch (IOException e) {
                Utils.showAlert("error", e.getMessage());
            }
        }
    }

    public void updateAction() {

    }

    public void deleteAction() {

    }
}
